/****************************************************************************
 chapter4.cpp
 
 This demo program displays a number of OpenGL primitive types, as well as
 how changing certain parameters can alter how the primitive is drawn.

 To change the primitive being displayed and the properties applied to it,
 use the following controls:

         Key  Action
         ---  ------
          1   Draw points
          2   Draw lines
          3   Draw triangles
          4   Draw a triangle strip
          5   Draw a triangle fan
          6   Draw quads
          7   Draw a polygon
          a   Toggle antialiasing
          s   Toggle stippling
          p   Rotate through polygon modes

  
 Author   :   Dave Astle
 Date     :   10/23/2000

 Written for OpenGL Game Programming
*****************************************************************************/


/********************************* Includes *********************************/
#define WIN32_LEAN_AND_MEAN				// trim the excess fat from Windows


#include <windows.h>					// standard Windows app include
#include <winuser.h>          // Windows constants
#include <gl/gl.h>						// standard OpenGL include
#include <gl/glu.h>						// OpenGL utilties
#include <stdlib.h>           // for random numbers


/*************************** Macros and Constants ***************************/
#define WND_CLASS_NAME  "OpenGL Window Class"


/******************************** Data types ********************************/
// used to track which primitive we're drawing
enum primtypes_t
{
  PT_POINT,
  PT_LINE,
  PT_TRIANGLE,
  PT_TRIANGLE_STRIP,
  PT_TRIANGLE_FAN,
  PT_QUAD,
  PT_POLYGON
};


/********************************* Globals **********************************/
// Windows variables
HDC       g_hdc;								  // global device context
HGLRC     g_hrc;                  // global rendering context
BOOL      g_isFullscreen = TRUE;  // toggles fullscreen and windowed display
BOOL      g_isActive = TRUE;      // false if window is minimized
HWND      g_hwnd = NULL;          // handle of our window  
HINSTANCE g_hInstance;            // application instance

// the current primitive
primtypes_t g_currentPrimitive = PT_POINT;

// used to hold data retrieved from OpenGL
GLfloat g_pointSize = 1.0;
GLfloat g_pointSizeStep;
GLfloat g_pointSizeMinMax[2];
GLfloat g_lineWidth = 1.0;
GLfloat g_lineWidthStep;
GLfloat g_lineWidthMinMax[2];

// the size of the window
GLuint g_width, g_height;


/******************************** Prototypes ********************************/
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
BOOL SetupWindow(char *title, int width, int height, int bits, bool isFullscreen);
BOOL InitializeScene();
GLvoid ResizeScene(GLsizei width, GLsizei height);
BOOL DisplayScene();
BOOL KillWindow();


/****************************************************************************
 WinMain()

 Main Windows entry point. Creates a window, sets up OpenGL, and enters an
 event loop.
*****************************************************************************/
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
  MSG   msg;       // message
  BOOL  isDone;    // flag saying when our app is complete

  // create our window and set up OpenGL
  SetupWindow("Primitives Demo", 800, 600, 32, TRUE);

  // intialize the loop condition variable
  isDone = FALSE;

  // main message loop
  while (!isDone)
  {
    // get a message if one is ready
    if(PeekMessage(&msg, g_hwnd, NULL, NULL, PM_REMOVE))
    {
      // look for a quit message, in which case, we're done
      if (msg.message == WM_QUIT)
      {
        isDone = TRUE;
      }
      else
      {
        // translate and dispatch to event queue
        TranslateMessage(&msg);
        DispatchMessage(&msg);
      }
    }

    // render the current scene
    DisplayScene();

    // bring backbuffer to foreground
    SwapBuffers(g_hdc);
  }

  // cleanup
  KillWindow();

  return msg.wParam;
} // end WinMain()


/****************************************************************************
 WndProc

 Windows message handler
*****************************************************************************/
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  static BOOL   antialias = false;    // controls antialiasing
  static GLenum polyMode  = GL_FILL;  // controls the polygon mode
  static BOOL   stipple   = false;    // controls stippling

  switch(message)
  {
  case WM_ACTIVATE:  // watch for the window being minimized and restored
    {
      if (!HIWORD(wParam))
      {
        // program was restored or maximized
        g_isActive = TRUE;
      }
      else
      {
        // program was minimized
        g_isActive=FALSE;
      }

      return 0;
    }
  case WM_SYSCOMMAND:  // look for screensavers and powersave mode
    {
      switch (wParam)
			{
				case SC_SCREENSAVE:     // screensaver trying to start
				case SC_MONITORPOWER:   // monitor going to powersave mode
          // returning 0 prevents either from happening
  				return 0;
        default:
          break;
      }
    } break;
  case WM_CLOSE:    // window is being closed
    {
      // send WM_QUIT to message queue
      PostQuitMessage(0);
      return 0;
    }
  case WM_SIZE:
    {
      // update perspective with new width and height
      ResizeScene(LOWORD(lParam), HIWORD(lParam));
      return 0;
    }
  case WM_CHAR:
    {
      switch (toupper(wParam))
      {
      case VK_ESCAPE:
        {
          // send WM_QUIT to message queue
          PostQuitMessage(0);
          return 0;
        }
      case 'A':   // toggle antialiasing
        {
          antialias = !antialias;

          // if antialiasing was just turned on, enable it
          if (antialias)
          {
            glEnable(GL_POINT_SMOOTH);
            glEnable(GL_LINE_SMOOTH);
            glEnable(GL_POLYGON_SMOOTH);
            glEnable(GL_BLEND);
          }
          // otherwise, disable it
          else
          {
            glDisable(GL_POINT_SMOOTH);
            glDisable(GL_LINE_SMOOTH);
            glDisable(GL_POLYGON_SMOOTH);
            glDisable(GL_BLEND);
          }
          return 0;
        }
      case 'S':   // toggle stippling
        {
          stipple = !stipple;

          // if stippling was just turned on, enable it
          if (stipple)
          {
            glEnable(GL_LINE_STIPPLE);
            glEnable(GL_POLYGON_STIPPLE);
          }
          // otherwise, disable it
          else
          {
            glDisable(GL_LINE_STIPPLE);
            glDisable(GL_POLYGON_STIPPLE);
          }
          return 0;
        }
      case 'P':   // rotate polygon mode
        {
          // go to the next mode
          polyMode++;

          // wrap around if we need to
          if (polyMode > GL_FILL)
            polyMode = GL_POINT;

          // select the new polygon mode
          glPolygonMode(GL_FRONT_AND_BACK, polyMode);
        }
      case '+':   // increase sizes
        {
          // increase point sizes (if we're not already at the max)
          if (g_pointSize < g_pointSizeMinMax[1])
            g_pointSize += g_pointSizeStep;

          glPointSize(g_pointSize);

          // increase line widths (if we're not already at the max)
          if (g_lineWidth < g_lineWidthMinMax[1])
            g_lineWidth += g_lineWidthStep;

          glLineWidth(g_lineWidth);

          return 0;
        }
      case '-':   // decrease sizes
        {
          // decrease point sizes (if we're not already at the min)
          if (g_pointSize > g_pointSizeMinMax[0])
            g_pointSize -= g_pointSizeStep;

          glPointSize(g_pointSize);

          // decrease line widths (if we're not already at the min)
          if (g_lineWidth > g_lineWidthMinMax[0])
            g_lineWidth -= g_lineWidthStep;

          glLineWidth(g_lineWidth);

          return 0;
        }
      case '1':   // select points
        {
          g_currentPrimitive = PT_POINT;
          return 0;
        }
      case '2':   // select lines
        {
          g_currentPrimitive = PT_LINE;
          return 0;
        }
      case '3':   // select triangles
        {
          g_currentPrimitive = PT_TRIANGLE;
          return 0;
        }
      case '4':   // select triangle strip
        {
          g_currentPrimitive = PT_TRIANGLE_STRIP;
          return 0;
        }
      case '5':   // select triangle fan
        {
          g_currentPrimitive = PT_TRIANGLE_FAN;
          return 0;
        }
      case '6':   // select quads
        {
          g_currentPrimitive = PT_QUAD;
          return 0;
        }
      case '7':   // select polygon
        {
          g_currentPrimitive = PT_POLYGON;
          return 0;   // select 
        }
      default:
        break;
      };
    } break;

  default:
    break;
  }

  // pass any remaining messages up to the default handler
  return (DefWindowProc(hwnd, message, wParam, lParam));
} // end WndProc()


/****************************************************************************
 SetupWindow()

 Creates a window and the device and rendering contexts for it.
*****************************************************************************/
BOOL SetupWindow(char *title, int width, int height, int bits, bool isFullscreen)
{
  // set the global flag
  g_isFullscreen = isFullscreen;

  // get our instance handle
  g_hInstance	= GetModuleHandle(NULL);

  WNDCLASSEX  wc;    // window class

  // fill out the window class structure
  wc.cbSize         = sizeof(WNDCLASSEX);
  wc.style          = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
  wc.lpfnWndProc    = WndProc;
  wc.cbClsExtra     = 0;
  wc.cbWndExtra     = 0;
  wc.hInstance      = g_hInstance;
  wc.hIcon          = LoadIcon(NULL, IDI_APPLICATION);	// default icon
  wc.hIconSm        = LoadIcon(NULL, IDI_WINLOGO);		  // windows logo small icon
  wc.hCursor        = LoadCursor(NULL, IDC_ARROW);		  // default arrow
  wc.hbrBackground  = NULL; //(HBRUSH) GetStockObject(BLACK_BRUSH);   // black background
  wc.lpszMenuName   = NULL;     // no menu
  wc.lpszClassName  = WND_CLASS_NAME;
  
  // register the windows class
  if (!RegisterClassEx(&wc))
  {
    MessageBox(NULL,"Unable to register the window class", "Error", MB_OK | MB_ICONEXCLAMATION);
    return FALSE;							// Exit And Return FALSE
  }

  // if we're in fullscreen mode, set the display up for it
  if (g_isFullscreen)
  {
    // set up the device mode structure
    DEVMODE screenSettings;
    memset(&screenSettings,0,sizeof(screenSettings));

    screenSettings.dmSize       = sizeof(screenSettings);	
    screenSettings.dmPelsWidth  = width;			// screen width
    screenSettings.dmPelsHeight = height;			// screen height
    screenSettings.dmBitsPerPel = bits;				// bits per pixel
    screenSettings.dmFields     = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;

    // attempt to switch to the resolution and bit depth we've selected
    if (ChangeDisplaySettings(&screenSettings, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
    {
      // if we can't get fullscreen, let them choose to quit or try windowed mode
      if (MessageBox(NULL, "Cannot run in the fullscreen mode at the selected resolution\n"
                           "on your video card. Try windowed mode instead?",
                           "OpenGL Game Programming",
                           MB_YESNO | MB_ICONEXCLAMATION) == IDYES)
      {
        g_isFullscreen = FALSE;	
      }
      else
			{
				return FALSE;
			}
    }
  }

	DWORD dwExStyle;
	DWORD dwStyle;

  // if we're still in fullscreen mode, set the window style appropriately
	if (g_isFullscreen)
	{
		dwExStyle = WS_EX_APPWINDOW;
		dwStyle = WS_POPUP;						// simple window with no borders or title bar
		ShowCursor(FALSE);            // hide the cursor for now
	}
	else
	{
		dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;
		dwStyle = WS_OVERLAPPEDWINDOW;
	}

  // set up the window we're rendering to so that the top left corner is at (0,0)
  // and the bottom right corner is (height,width)
  RECT  windowRect;
  windowRect.left = 0;
  windowRect.right = (LONG) width;
  windowRect.top = 0;
  windowRect.bottom = (LONG) height;

  // change the size of the rect to account for borders, etc. set by the style
  AdjustWindowRectEx(&windowRect, dwStyle, FALSE, dwExStyle);

  // class registered, so now create our window
  g_hwnd = CreateWindowEx(dwExStyle,          // extended style
                          WND_CLASS_NAME,     // class name
                          title,              // app name
                          dwStyle |           // window style
                          WS_CLIPCHILDREN |   // required for
                          WS_CLIPSIBLINGS,    // using OpenGL
                          0, 0,               // x,y coordinate
                          windowRect.right - windowRect.left, // width
                          windowRect.bottom - windowRect.top, // height
                          NULL,               // handle to parent
                          NULL,               // handle to menu
                          g_hInstance,        // application instance
                          NULL);              // no extra params

  // see if our window handle is valid
  if (!g_hwnd)
	{
    // reset the display
		KillWindow();
		MessageBox(NULL, "Unable to create window", "Error", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}

  // get a device context
	if (!(g_hdc = GetDC(g_hwnd)))
	{
    // reset the display
		KillWindow();
		MessageBox(NULL,"Unable to create device context", "Error", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}

  // set the pixel format we want
  PIXELFORMATDESCRIPTOR pfd = {
    sizeof(PIXELFORMATDESCRIPTOR),	// size of structure
    1,                              // default version
    PFD_DRAW_TO_WINDOW |            // window drawing support
    PFD_SUPPORT_OPENGL |            // OpenGL support
    PFD_DOUBLEBUFFER,               // double buffering support
    PFD_TYPE_RGBA,                  // RGBA color mode
    bits,                           // 32 bit color mode
    0, 0, 0, 0, 0, 0,               // ignore color bits, non-palettized mode
    0,                              // no alpha buffer
    0,                              // ignore shift bit
    0,                              // no accumulation buffer
    0, 0, 0, 0,                     // ignore accumulation bits
    16,                             // 16 bit z-buffer size
    0,                              // no stencil buffer
    0,                              // no auxiliary buffer
    PFD_MAIN_PLANE,                 // main drawing plane
    0,                              // reserved
    0, 0, 0 };                      // layer masks ignored
      
	GLuint  pixelFormat;

	// choose best matching pixel format
	if (!(pixelFormat = ChoosePixelFormat(g_hdc, &pfd)))
	{
    // reset the display
		KillWindow();
		MessageBox(NULL, "Can't find an appropriate pixel format", "Error", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}

	// set pixel format to device context
  if(!SetPixelFormat(g_hdc, pixelFormat,&pfd))
	{
    // reset the display
		KillWindow();
		MessageBox(NULL, "Unable to set pixel format", "Error", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
  }

  // create the OpenGL rendering context
  if (!(g_hrc = wglCreateContext(g_hdc)))
	{
    // reset the display
		KillWindow();
		MessageBox(NULL, "Unable to create OpenGL rendering context", "Error",MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}

  // now make the rendering context the active one
  if(!wglMakeCurrent(g_hdc, g_hrc))
	{
    // reset the display
		KillWindow();
		MessageBox(NULL,"Unable to activate OpenGL rendering context", "ERROR", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}

  // show the window in the forground, and set the keyboard focus to it
  ShowWindow(g_hwnd, SW_SHOW);
	SetForegroundWindow(g_hwnd);
	SetFocus(g_hwnd);

  // set up the perspective for the current screen size
	ResizeScene(width, height);

  // do one-time initialization
  if (!InitializeScene())
	{
    // reset the display
		KillWindow();
		MessageBox(NULL, "Initialization failed", "Error", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}

  return TRUE;
} // end SetupWindow()


/****************************************************************************
 InitializeScene()

 Sets up everything for our scene that only needs to be done once.
*****************************************************************************/
BOOL InitializeScene()
{
  // use Gouraud shading
  glShadeModel(GL_SMOOTH);

  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  // tell OpenGL to use the nicest built-in antialiasing possible
  glHint(GL_POINT_SMOOTH, GL_NICEST);
  glHint(GL_LINE_SMOOTH, GL_NICEST);
  glHint(GL_POLYGON_SMOOTH, GL_NICEST);

  // get the min and max point sizes and the step size
  glGetFloatv(GL_POINT_SIZE_RANGE, g_pointSizeMinMax);
  glGetFloatv(GL_POINT_SIZE_GRANULARITY, &g_pointSizeStep);

  // get the min and max line widths and the step size
  glGetFloatv(GL_LINE_WIDTH_RANGE, g_lineWidthMinMax);
  glGetFloatv(GL_LINE_WIDTH_GRANULARITY, &g_lineWidthStep);

  // set a line stipple pattern, setting each bit to repeat 3 times
  GLushort lineStipple = 0xEB32;
  glLineStipple(3, lineStipple);

  // specify a polygon mask for stippling. This is a rough copy of the
  // OpenGL logo
  GLubyte mask[] = {
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x7, 0xFF, 0xF0, 0x0,
    0x1F, 0xE0, 0x1E, 0x0,
    0x7E, 0x20, 0x01, 0x0,
    0x7F, 0x39, 0xAB, 0x6F,
    0xFC, 0xA6, 0xAB, 0x6F,
    0xF8, 0x66, 0x6E, 0xEC,
    0xF8, 0x6A, 0x2A, 0x6C,
    0xFC, 0x80, 0x02, 0x2C,
    0x7F, 0x0, 0x1, 0xC8,
    0x7F, 0x0, 0x1, 0xC8,
    0x1F, 0xC0, 0x0C, 0x0,
    0x07, 0xFF, 0xF0, 0x0,
    0x03, 0xFF, 0xE0, 0x0,
    0x0, 0x1C, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0,
    0x0, 0x0, 0x0, 0x0
  };

  // tell OpenGL to use this mask for stippling
  glPolygonStipple(mask);

  return TRUE;
} // end InitializeScene()


/****************************************************************************
 Resize()

 Sets up the correct perspective based on the current screen size.
*****************************************************************************/
GLvoid ResizeScene(GLsizei width, GLsizei height)
{
  // reset the viewport to the new dimensions
  glViewport(100, 100, width-100, height-100);

  // set our globals to match the (possibly new) values
  g_width = width;
  g_height = height;

  // select the projection matrix and clear it out
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  // set the perspective
  gluOrtho2D(0, width, 0, height);

  // select modelview matrix and clear it out
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
} // end ResizeScene()


/****************************************************************************
 DisplayScene()

 Actually draw our scene. This will be called every frame.
*****************************************************************************/
BOOL DisplayScene()
{
  // clear the screen
  glClear(GL_COLOR_BUFFER_BIT);

  // we're using random numbers for most of the colors and primitive
  // positions. Because we want to have the random numbers chosen every frame,
  // we need to seed the random number generator with the same number every
  // frame. Try commenting this line out to see what happens.
  srand(g_currentPrimitive);

  // only draw the currently selected primitive
  switch(g_currentPrimitive)
  {
  case PT_POINT:
    {
      // draw 40 random points. Note that we're only making a single call to
      // glBegin().
      glBegin(GL_POINTS);
        for (int n = 0; n < 40; n++)
        {
          glColor3ub(rand() % 256, rand() % 256, rand() % 256);
          glVertex2i(rand() % g_width, rand() % g_height);
        }
      glEnd();

    } break;
  case PT_LINE:
    {
      // draw 10 random lines. Each pair of vertices specifies a new line
      glBegin(GL_LINES);
        for (int n = 0; n < 10; n++)
        {
          glColor3ub(rand() % 256, rand() % 256, rand() % 256);
          glVertex2i(rand() % g_width, rand() % g_height);
          glColor3ub(rand() % 256, rand() % 256, rand() % 256);
          glVertex2i(rand() % g_width, rand() % g_height);
        }
      glEnd();
    } break;
  case PT_TRIANGLE:
    {
      // draw 3 random triangles
      glBegin(GL_TRIANGLES);
        for (int n = 0; n < 3; n++)
        {
          glColor3ub(255, rand() % 256, rand() % 256);
          glVertex2i(rand() % g_width, rand() % g_height);
          glColor3ub(rand() % 256, 255, rand() % 256);
          glVertex2i(rand() % g_width, rand() % g_height);
          glColor3ub(rand() % 256, rand() % 256, 255);
          glVertex2i(rand() % g_width, rand() % g_height);
        }
      glEnd();

    } break;
  case PT_TRIANGLE_STRIP:
    {
      // draw a single triangle strip
      glBegin(GL_TRIANGLE_STRIP);
        glColor3ub(255, 0, 0);
        glVertex2i(10, 10);
        glColor3ub(0, 255, 0);
        glVertex2i(100, 50);
        glColor3ub(0, 0, 255);
        glVertex2i(75, 120);
        glColor3ub(255, 0, 0);
        glVertex2i(150, 110);
        glColor3ub(0, 255, 0);
        glVertex2i(200, 180);
        glColor3ub(0, 0, 255);
        glVertex2i(400, 300);
        glColor3ub(255, 0, 0);
        glVertex2i(350, 400);
        glColor3ub(0, 255, 0);
        glVertex2i(470, 275);
        glColor3ub(0, 0, 255);
        glVertex2i(600, 480);
      glEnd();
    } break;
  case PT_TRIANGLE_FAN:
    {
      // draw a single triangle fan. Notice that this one is closed
      // (the last vertex is the same as the second) but it doesn't
      // have to be
      glBegin(GL_TRIANGLE_FAN);
        glColor3ub(255, 255, 255);
        glVertex2i(400, 300);
        glColor3ub(255, 0, 0);
        glVertex2i(320, 310);
        glColor3ub(0, 255, 0);
        glVertex2i(300, 250);
        glColor3ub(0, 0, 255);
        glVertex2i(330, 220);
        glColor3ub(255, 0, 0);
        glVertex2i(340, 190);
        glColor3ub(0, 255, 0);
        glVertex2i(390, 180);
        glColor3ub(0, 0, 255);
        glVertex2i(430, 200);
        glColor3ub(255, 0, 0);
        glVertex2i(475, 285);
        glColor3ub(0, 255, 0);
        glVertex2i(510, 330);
        glColor3ub(0, 0, 255);
        glVertex2i(460, 380);
        glColor3ub(255, 0, 0);
        glVertex2i(405, 415);
        glColor3ub(0, 255, 0);
        glVertex2i(375, 375);
        glColor3ub(0, 0, 255);
        glVertex2i(350, 365);
        glColor3ub(255, 0, 0);
        glVertex2i(320, 310);
      glEnd();
    } break;
  case PT_QUAD:
    {
      // draw 3 quads. I'm not doing these randomly because it'd be
      // difficult to ensure that the polygons are convex and that
      // edges don't cross
      glBegin(GL_QUADS);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(60, 60);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(200, 80);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(210, 190);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(70, 200);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(500, 420);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(630, 450);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(635, 460);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(495, 500);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(190, 180);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(500, 190);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(400, 350);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(300, 300);
      glEnd();
    } break;
  case PT_POLYGON:
    {
      // draw a single polygon. I'm not doing this randomly for
      // the same reason quads aren't done randomly.
      glBegin(GL_POLYGON);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(390, 420);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(300, 400);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(240, 300);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(250, 250);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(370, 200);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(480, 220);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(500, 290);
        glColor3ub(rand() % 256, rand() % 256, rand() % 256);
        glVertex2i(520, 400);
      glEnd();
    } break;
  }

  return TRUE;
} // end DisplayScene()


/****************************************************************************
 KillWindow()

 Restore the original display and dispose of the window, device context, and
 rendering context properly.
*****************************************************************************/
BOOL KillWindow()
{
  // restore the original display if we're in fullscreen mode
  if (g_isFullscreen)
  {
    ChangeDisplaySettings(NULL, 0);
    ShowCursor(TRUE);
  }

  // if we have an RC, release it
  if (g_hrc)
  {
    // release the RC
    if (!wglMakeCurrent(NULL,NULL))
    {
      MessageBox(NULL, "Unable to release rendering context", "Error", MB_OK | MB_ICONINFORMATION);
    }

    // delete the RC
    if (!wglDeleteContext(g_hrc))
    {
      MessageBox(NULL, "Unable to delete rendering context", "Error", MB_OK | MB_ICONINFORMATION);
    }

    g_hrc = NULL;
  }

  // release the DC if we have one
  if (g_hdc && !ReleaseDC(g_hwnd, g_hdc))
  {
    MessageBox(NULL, "Unable to release device context", "Error", MB_OK | MB_ICONINFORMATION);
    g_hdc = NULL;
  }

  // destroy the window if we have a valid handle
  if (g_hwnd && !DestroyWindow(g_hwnd))
  {
    MessageBox(NULL, "Unable to destroy window", "Error", MB_OK | MB_ICONINFORMATION);
    g_hwnd = NULL;
  }

  // unregister our class so we can create a new one if we need to
  if (!UnregisterClass(WND_CLASS_NAME, g_hInstance))
  {
    MessageBox(NULL, "Unable to unregister window class", "Error", MB_OK | MB_ICONINFORMATION);
    g_hInstance = NULL;
  }

  return TRUE;
} // end KillWindow()


